import {
  addLearningObjective,
  updateLearningObjective,
} from '@/editor/indexeddb/operations'
import type { ButtonProps } from '@blueprintjs/core'
import {
  Button,
  Dialog,
  DialogBody,
  DialogFooter,
  InputGroup,
  Label,
} from '@blueprintjs/core'
import { useNotifyContext } from '@inject/shared/notification/contexts/NotifyContext'
import type { FC } from 'react'
import { memo, useCallback, useEffect, useState } from 'react'
import type { LearningObjectiveInfo } from '../indexeddb/types'

interface LearningObjectiveFormProps {
  learningObjective?: LearningObjectiveInfo
  buttonProps: ButtonProps
}

const LearningObjectiveForm: FC<LearningObjectiveFormProps> = ({
  learningObjective,
  buttonProps,
}) => {
  const [isOpen, setIsOpen] = useState(false)
  const [name, setName] = useState<string>('')
  const { notify } = useNotifyContext()

  const clearInput = useCallback(() => {
    setName('')
  }, [])

  const handleAddButton = useCallback(
    async (objective: Omit<LearningObjectiveInfo, 'id'>) => {
      try {
        await addLearningObjective(objective)
        clearInput()
        setIsOpen(false)
      } catch (err) {
        notify(`Failed to add learning objective '${objective.name}': ${err}`, {
          intent: 'danger',
        })
      }
    },
    [notify]
  )

  const handleUpdateButton = useCallback(
    async (objective: LearningObjectiveInfo) => {
      try {
        await updateLearningObjective(objective)
        setIsOpen(false)
      } catch (err) {
        notify(
          `Failed to update learning objective '${objective.name}': ${err}`,
          {
            intent: 'danger',
          }
        )
      }
    },
    [notify]
  )

  useEffect(() => {
    setName(learningObjective?.name || '')
  }, [learningObjective])

  return (
    <>
      <Button {...buttonProps} onClick={() => setIsOpen(true)} />
      <Dialog
        isOpen={isOpen}
        onClose={() => setIsOpen(false)}
        icon={learningObjective ? 'edit' : 'plus'}
        title={
          learningObjective
            ? 'Edit learning objective'
            : 'New learning objective'
        }
      >
        <DialogBody>
          <Label>
            Title
            <InputGroup
              placeholder='Input text'
              value={name}
              onChange={e => setName(e.target.value)}
            />
          </Label>
        </DialogBody>
        <DialogFooter
          actions={
            learningObjective ? (
              <Button
                disabled={!name}
                onClick={() =>
                  handleUpdateButton({
                    id: learningObjective.id,
                    name,
                  })
                }
                intent='primary'
                icon='edit'
                text='Save changes'
              />
            ) : (
              <Button
                disabled={!name}
                onClick={() => handleAddButton({ name })}
                intent='primary'
                icon='plus'
                text='Add'
              />
            )
          }
        />
      </Dialog>
    </>
  )
}

export default memo(LearningObjectiveForm)
