import EmailContactSelector from '@/email/EmailContactSelector'
import EmailTemplates from '@/email/EmailTemplates'
import SenderSelector from '@/email/SenderSelector'
import { Button, InputGroup } from '@blueprintjs/core'
import { css } from '@emotion/css'
import type { EmailParticipant } from '@inject/graphql/fragments/EmailParticipant.generated'
import type { EmailTemplate } from '@inject/graphql/fragments/EmailTemplate.generated'
import type { EmailThread } from '@inject/graphql/fragments/EmailThread.generated'
import type { FileInfo } from '@inject/graphql/fragments/FileInfo.generated'
import notEmpty from '@inject/shared/utils/notEmpty'
import type { Dispatch, FC, SetStateAction } from 'react'
import { useEffect, useMemo, useState } from 'react'

const gridContainer = css`
  display: grid;
  grid-template-columns: auto 1fr;
  grid-column-gap: 0.5rem;
`

const gridItem = css`
  display: flex;
  align-items: center;
`

const contactSelectorWrapper = css`
  width: 100%;
  display: flex;
`

const contactSelector = css`
  flex: 1;
`

type HeaderAreaProps = (
  | {
      // new thread
      emailThread?: undefined
      selectedContacts: string[]
      setSelectedContacts: Dispatch<SetStateAction<string[]>>
      bccSelectedContacts: string[]
      setBccSelectedContacts: Dispatch<SetStateAction<string[]>>
      subject: string | undefined
      setSubject: Dispatch<SetStateAction<string | undefined>>
    }
  | {
      // existing thread
      emailThread: EmailThread
      selectedContacts?: undefined
      setSelectedContacts?: undefined
      bccSelectedContacts?: undefined
      setBccSelectedContacts?: undefined
      subject?: undefined
      setSubject?: undefined
    }
) &
  (
    | {
        // instructor
        senderList: string[]
        setSenderAddress: Dispatch<SetStateAction<string>>
        setContent: Dispatch<SetStateAction<string>>
        setActivateMilestone: Dispatch<SetStateAction<string>>
        setDeactivateMilestone: Dispatch<SetStateAction<string>>
        setFileInfo: Dispatch<SetStateAction<FileInfo | undefined>>
        template: EmailTemplate | undefined
        setTemplate: Dispatch<SetStateAction<EmailTemplate | undefined>>
        teamId: string
        senderAddress: string
        teamAddress?: never
      }
    | {
        // trainee
        senderList?: undefined
        setSenderAddress?: undefined
        setContent?: undefined
        setActivateMilestone?: undefined
        setDeactivateMilestone?: undefined
        setFileInfo?: undefined
        template?: undefined
        setTemplate?: undefined
        teamId?: undefined
        senderAddress?: never
        teamAddress: string
      }
  ) & {
    contacts: EmailParticipant[]
    exerciseId: string
  }

const HeaderArea: FC<HeaderAreaProps> = ({
  senderAddress,
  emailThread,
  senderList,
  contacts,
  selectedContacts,
  setSelectedContacts,
  bccSelectedContacts,
  setBccSelectedContacts,
  subject,
  setSubject,
  setSenderAddress,
  setContent,
  setActivateMilestone,
  setDeactivateMilestone,
  setFileInfo,
  template,
  setTemplate,
  teamId,
  exerciseId,
  teamAddress,
}) => {
  const emailThreadParticipants = useMemo(
    () =>
      (emailThread?.participants || [])
        .filter(notEmpty)
        .map(p => p.address)
        .join(', '),
    [emailThread]
  )

  useEffect(() => {
    if (senderList && senderList.length === 1) {
      setSenderAddress && setSenderAddress(senderList[0])
    }
    if (senderList && senderList.length === 0) {
      setSenderAddress && setSenderAddress('')
    }
  }, [senderList, setSenderAddress])

  const [bccOpen, setBccOpen] = useState(false)

  return (
    <div className={gridContainer}>
      <div className={gridItem}>From:</div>
      {senderList ? (
        <SenderSelector
          emailContacts={contacts}
          senderList={senderList}
          senderAddress={senderAddress}
          setSenderAddress={setSenderAddress}
        />
      ) : (
        <InputGroup readOnly value={teamAddress} />
      )}

      <div className={gridItem}>Participants:</div>
      {emailThread ? (
        <InputGroup readOnly value={emailThreadParticipants} />
      ) : (
        <div className={contactSelectorWrapper}>
          <div className={contactSelector}>
            <EmailContactSelector
              placeholder='Add participants...'
              emailContacts={contacts}
              selectedContacts={selectedContacts}
              setSelectedContacts={setSelectedContacts}
              inInstructor={senderList !== undefined}
              exerciseId={exerciseId}
              teamAddress={teamAddress}
            />
          </div>
          <Button
            minimal
            outlined
            active={bccOpen}
            onClick={() => {
              setBccOpen(prev => !prev)
              if (!bccOpen) {
                setBccSelectedContacts([])
              }
            }}
          >
            Bcc
          </Button>
        </div>
      )}

      {!emailThread && bccOpen && (
        <>
          <div className={gridItem}>Bcc:</div>
          <EmailContactSelector
            placeholder='Add blind copy recipients...'
            emailContacts={contacts}
            selectedContacts={bccSelectedContacts}
            setSelectedContacts={setBccSelectedContacts}
            inInstructor={senderList !== undefined}
            exerciseId={exerciseId}
          />
        </>
      )}

      <div className={gridItem}>Subject:</div>
      {emailThread ? (
        <InputGroup readOnly value={emailThread.subject} />
      ) : (
        <InputGroup
          placeholder='Add a subject...'
          value={subject || ''}
          onChange={event => setSubject(event.currentTarget.value)}
        />
      )}

      {senderList !== undefined && (
        <>
          <div className={gridItem}>Template:</div>
          <EmailTemplates
            setSenderAddress={setSenderAddress}
            setContent={setContent}
            setActivateMilestone={setActivateMilestone}
            setDeactivateMilestone={setDeactivateMilestone}
            setFileInfo={setFileInfo}
            template={template}
            setTemplate={setTemplate}
            teamId={teamId}
            exerciseId={exerciseId}
            {...(emailThread
              ? { threadId: emailThread.id }
              : { emailAddresses: senderList, setSubject })}
          />
        </>
      )}
    </div>
  )
}

export default HeaderArea
