import {
  Button,
  Checkbox,
  Dialog,
  DialogBody,
  FormGroup,
  InputGroup,
  Spinner,
} from '@blueprintjs/core'
import { httpHello, wsGraphql } from '@inject/shared/config'
import type { FC, ReactNode } from 'react'
import { useEffect, useState } from 'react'
import { host, useHost } from './host'
import { useWs, ws } from './ws'

const Loading = () => (
  <div
    style={{
      height: '100%',
      width: '100%',
      display: 'flex',
      gap: '0.5rem',
      flexDirection: 'column',
      justifyContent: 'center',
      alignItems: 'center',
    }}
  >
    <Spinner intent='primary' />
  </div>
)

interface QuestionDialogInterface {
  trigger: (host: string, ws?: string) => void
  host: string
  ws: string
}

const QuestionDialog: FC<QuestionDialogInterface> = ({
  trigger,
  host: host_,
  ws: ws_,
}) => {
  const [open, setOpen] = useState(true)
  const Form = () => {
    const [host, setHost] = useState<string>(host_)
    const [ws, setWs] = useState<string>(ws_)
    const [customWs, setCustomWs] = useState(false)
    return (
      <form
        onSubmit={e => {
          e.preventDefault()
          if (customWs) trigger(host, ws)
          else trigger(host)
          setOpen(false)
        }}
      >
        <FormGroup label='Host:'>
          <InputGroup
            typeof='text'
            defaultValue={host_}
            placeholder='localhost'
            onChange={e => setHost(e.target.value)}
          />
        </FormGroup>
        <Checkbox
          label='Use custom WebSocket (relevant on Nginx Deployments)'
          checked={customWs}
          onClick={() => setCustomWs(prev => !prev)}
        />
        {customWs && (
          <FormGroup label='Websocket Server:'>
            <InputGroup
              typeof='text'
              defaultValue={ws_}
              placeholder='ws://localhost/inject/api/v1/subscription'
              onChange={e => setWs(e.target.value)}
            />
          </FormGroup>
        )}
        <Button fill type='submit' intent='primary'>
          Reconnect
        </Button>
      </form>
    )
  }

  return (
    <Dialog
      style={{
        margin: 0,
        maxWidth: '90vw',
        maxHeight: '90vh',
      }}
      title='Connection error'
      icon='error'
      isOpen={open}
      onClose={() => setOpen(false)}
      canOutsideClickClose={false}
      canEscapeKeyClose={false}
      isCloseButtonShown={false}
    >
      <DialogBody>
        <p>
          Unable to connect to server located at <code>{host_}</code>, please
          type in a new address
        </p>
        <Form />
      </DialogBody>
    </Dialog>
  )
}

interface ConnectionProps {
  host: string
  ws?: string
  children: ReactNode
}

enum ConnectionStatus {
  CONNECTED,
  FAILED,
  CONNECTING,
}

const Connection: FC<ConnectionProps> = ({
  host: host_,
  ws: ws_,
  children,
}) => {
  const hostVar = useHost()
  const wsVar = useWs()

  const [state, setState] = useState<ConnectionStatus>(
    ConnectionStatus.CONNECTING
  )

  useEffect(() => {
    if (state === ConnectionStatus.CONNECTED && (!hostVar || !wsVar)) {
      // resets back to connecting state if host or ws is not set
      setState(ConnectionStatus.CONNECTING)
      return
    }
    /*
     * If the state is connected, we don't need to do anything
     * If the state is connecting, we need to check if the host is set
     * If the state is failed, we need to show the dialog
     */
    switch (state) {
      case ConnectionStatus.CONNECTED:
        console.log('Connected to backend!')
        break
      case ConnectionStatus.CONNECTING:
        /*
         * Connection initialization
         * populates ws and host if not set
         * checks if the server is reachable
         * sets the state to connected if successful
         * sets the state to failed if unsuccessful
         */
        if (!wsVar) {
          if (!ws_) {
            ws(wsGraphql(hostVar || host_))
          } else {
            ws(ws_)
          }
        }
        if (!hostVar) {
          host(host_)
        }
        fetch(httpHello(hostVar || host_), {
          credentials: 'include',
          method: 'GET',
        })
          .then(() => {
            setState(ConnectionStatus.CONNECTED)
          })
          .catch(() => {
            setState(ConnectionStatus.FAILED)
          })
        break
      case ConnectionStatus.FAILED:
        /*
         * Connection failed
         * shows the dialog
         */
        break
    }
  }, [state, host_, ws_, hostVar, wsVar])

  return (
    <>
      {ConnectionStatus.CONNECTED === state && children}
      {ConnectionStatus.FAILED === state && (
        <QuestionDialog
          host={hostVar || host_}
          ws={wsVar || ws_ || wsGraphql(hostVar || host_)}
          trigger={(_host, _ws) => {
            setState(ConnectionStatus.CONNECTING)
            host(_host)
            if (_ws) {
              ws(_ws)
            } else {
              ws(wsGraphql(_host))
            }
          }}
        />
      )}
      {ConnectionStatus.CONNECTING === state && <Loading />}
    </>
  )
}

export default Connection
